/*
* bb_util.h - headers for utility procedures for blockbuilder.
*/

#ifndef BB_UTIL
#define BB_UTIL

#include "util.h"
#include "seq.h"

enum {
  BUF_SIZE=500
};

/* a sequence (species) is a set of contigs */
typedef struct seq_ {
	char *name;
	int ncontig;
} Seq, *Seq_ptr;

/* a contig has a header, a length, and a string of nucleotides */
typedef struct contig_ {
	int seq_nbr;
	char *header;
	int len;
	uchar *chars;
} Contig, *Contig_ptr;

/* A row of a block contains a "segment" consisting of a contiguous range of
*  positions in a contig or the reverse complement of a contig, possibly with
*  interspersed dashes.
*/
typedef struct seg_ {
	int contig_nbr, beg, end;
	uchar *chars;
} Seg, *Seg_ptr;

/* a block is a rectangular array whose rows are segments; the blocks are in a
*  simple linked list */
typedef struct block_ {
	int nrow, ncol, mark;
	Seg_ptr seg;
	struct block_ *next;
} Block, *Block_ptr;

/* print human-readable form of one block or of all blocks */
void show_block(Block_ptr b, FILE *debug_file);
void show_blocks(Block_ptr b, FILE *debug_file);

/* convert a column in a block to a position in a segment, and vice versa,
*  accounting for dashes in the segment */
int col_of_pos(int pos, Seg_ptr s, int slen);
int pos_of_col(int col, Seg_ptr s, int slen);

/* find a block's segment from a given contig */
Seg_ptr find_seg(int contig_nbr, int pos, Contig_ptr contig, Block_ptr b,
int must);

/* walk along the blocks threaded by a contig */
Block_ptr find_block(Block_ptr block_list, int contig_nbr, int pos,
  Contig_ptr contig, Seg_ptr *S);
Block_ptr adjacent_block(Seg_ptr *S, Contig_ptr contig, Block_ptr blocks, int forward);
Block_ptr prev_block(Seg_ptr *S, Contig_ptr contig, Block_ptr blocks);
Block_ptr next_block(Seg_ptr *S, Contig_ptr contig, Block_ptr blocks);

/* input and output of all blocks */
void write_blocks(Block_ptr block_list, Seq_ptr seq, int nseq,
  Contig_ptr contig, int ncontig, int verbose);
Block_ptr read_blocks(char *filename, Seq_ptr *Seq, int *Nseq, Contig_ptr *,
  int *Ncontig);

Block_ptr block_excise(Block_ptr b, Block_ptr blocks);
void block_free(Block_ptr b);
void reduce_block(Block_ptr b, Contig_ptr contig);
int get_row_from_seq_num(int seq_num, int nrow, Block_ptr block);

#endif

